\name{edbQuery.RODBC_MySQL}

\alias{edbQuery.RODBC_MySQL}

\title{Send and retrieve a query in a SQLite database (referenced by 'edb').}

\description{Send and retrieve a query in a SQLIte database (referenced by 'edb'). }

\usage{\method{edbQuery}{RODBC_MySQL}(edb, statement, formatCol = NULL, 

    verbose = FALSE, ...)}

\arguments{

  \item{edb}{An object of class 'edb', such as returned by \code{\link[easydb]{edb}}.}

  \item{statement}{Single character string. SQL statement / SQL query to be passed 

to \code{link[RODBC]{sqlQuery}}.}

  \item{formatCol}{If not NULL, a named list of functions to be applied to certain columns 

after the data has been extracted from the database. The name of each list 

item gives the column to process, and the value of each item gives the 

function that must be applied. For instance 

formatCol = list("DATE"=as.Date) will apply the function 

\link[base]{as.Date} to the column "DATE".}

  \item{verbose}{Single logical. If TRUE, information on what is done are output 

on screen.}

  \item{\dots}{Additional parameters to be passed to \code{link[RODBC]{sqlQuery}}.}

}



\value{The function returns the requested table. }



\author{Julien MOEYS <Julien.Moeys@mark.slu.se>}







\seealso{\code{\link[easydb]{edb}}, \code{link[RODBC]{sqlQuery}}.}

\examples{library( "easyrodbcmysql" ) 







testMySQL <- FALSE # Set to TRUE to run some tests

                   # after modifying the edb() part below 

                   # with your own database setting







### Windows only:

if( (Sys.info()[[ "sysname" ]] == "Windows") & testMySQL ){ 

    

    ### Describe the database (NB: this is not a connection)

    myDb <- edb( 

        dbType       = "RODBC_MySQL", 

        dbSourceName = "nameOfODBCSource", # or any name you like

        dbName       = "nameOfDatabase", 

        dbLogin      = "yourUserName", 

        dbPwd        = "yourPassword", 

        dbHost       = "127.0.0.1", 

        dbPort       = 3306 

    )   #    

    

    

    

    ### Register the data source in ODBC 

    edbDataSource( myDb, verbose = TRUE ) 

    

    

    

    ### Use the database:

    

    ## Read data in a table in the database

    

    # Retrieve a table (data.frame style subsetting):

    myDb[ "WRB_SOIL_GROUP" ] 

    

    # Same operation, but with edbRead()

    edbRead( edb = myDb, tableName = "WRB_SOIL_GROUP" ) 

    

    # Retrieve part of a table (with row constrains)

    myDb[ "WRB_SOIL_GROUP", list("ABBREV" = c("AC","CR","PL")), 

        verbose = TRUE ] 

    

    # Same operation, but with edbQuery() 

    edbQuery( edb = myDb, statement = 

        "SELECT * FROM [WRB_SOIL_GROUP] WHERE [ABBREV] IN ('AC','CR','PL')" ) 

    

    ### Un-register the data source in ODBC (windows only)

    edbDataSource( myDb, trash = TRUE ) 

}   #

}

